import 'package:fling_pickle/fling_pickle.dart';
import 'package:test/test.dart';

void main() {
  group('asynchronous SerDes', () {
    test('single-level pickle', () async {
      // given
      final pickler = BinaryPickler();
      final pickle = PickleBuilder().withBool('success', true).build();

      // when
      final result = await pickler.read(pickler.write(pickle));

      // then
      expect(result.readBool('success'), true);
    });
    test('nested pickle', () async {
      // given
      final pickler = BinaryPickler();
      final pickle = PickleBuilder()
          .withInt('int', 3)
          .withString('string', 'value')
          .withPickle(
              'pickle', PickleBuilder().withBool('nestedBool', true).build())
          .build();

      // when
      final result = await pickler.read(pickler.write(pickle));

      // then
      expect(result.readPickle('pickle').readBool('nestedBool'), true);
    });
    test('list', () async {
      // given
      final pickler = BinaryPickler();
      final pickle = PickleBuilder().withPickles('pickles', []).build();

      // when
      final result = await pickler.read(pickler.write(pickle));

      // then
      expect(result.readPickles('pickles'), []);
    });
  });

  group('synchronous SerDes', () {
    test('single-level pickle', () {
      // given
      final pickler = BinaryPickler();
      final pickle = PickleBuilder().withBool('success', true).build();

      // when
      final result = pickler.readSync(pickler.writeSync(pickle));

      // then
      expect(result.readBool('success'), true);
    });
    test('nested pickle', () {
      // given
      final pickler = BinaryPickler();
      final pickle = PickleBuilder()
          .withInt('int', 3)
          .withString('string', 'value')
          .withPickle(
              'pickle', PickleBuilder().withBool('nestedBool', true).build())
          .build();

      // when
      final result = pickler.readSync(pickler.writeSync(pickle));

      // then
      expect(result.readPickle('pickle').readBool('nestedBool'), true);
    });
    test('list', () {
      // given
      final pickler = BinaryPickler();
      final pickle = PickleBuilder().withPickles('pickles', []).build();

      // when
      final result = pickler.readSync(pickler.writeSync(pickle));

      // then
      expect(result.readPickles('pickles'), []);
    });
  });
}
